import 'package:flutterskel/provider/ProviderBuilder.dart' as arch;

import 'configuration/AppConfiguration.dart';
import 'data/sources/DataSource.dart';
import 'resources/icons/AppIcons.dart';
import 'resources/images/AppImages.dart';
import 'resources/strings/AppStrings.dart';
import 'resources/themes/AppTheme.dart';
import 'resources/themes/ThemeRepository.dart';
import 'routing/page/PageRouter.dart';
import 'routing/url/UrlRouter.dart';
import 'userinformation/UserInformationRepository.dart';



class ProviderBuilder
    extends arch.ProviderBuilder<
        DataSource,
        // Routing
        UrlRouter,
        PageRouter,
        // Configuration
        AppConfiguration,
        // UserInformation
        UserInformationRepository,
        // Theme
        ThemeRepository,
        // Strings
        AppStrings,
        // Images and Icons
        AppImages,
        AppIcons> {


  @override
  DataSource buildDataSource() => DataSource();

  @override
  ThemeRepository buildThemeRepository() {
    var theme = AppTheme();
    return ThemeRepository(theme: theme);
  }

  @override
  AppIcons buildIcons() => AppIcons();

  @override
  UserInformationRepository buildUserInformationRepository() {
    return UserInformationRepository();
  }

  @override
  AppStrings buildStringsResources() {
    return AppStrings();
  }

  @override
  AppImages buildImages() {
    return AppImages();
  }

  @override
  AppConfiguration buildConfiguration() {
    return AppConfiguration();
  }

  @override
  UrlRouter buildUrlRouter() {
    return UrlRouter();
  }

  @override
  PageRouter buildPageRouter() {
    return PageRouter();
  }
}