import 'package:flutter/widgets.dart';
import 'package:flutterskel/configuration/Configuration.dart' as arch;
import 'package:flutterskel/resources/strings/StringResources.dart' as arch;
import 'package:flutterskel/resources/strings/StringsRepository.dart';
import 'package:flutterskel/resources/themes/ThemeRepository.dart' as arch;
import 'package:flutterskel/routing/page/PageRouter.dart' as arch;
import 'package:flutterskel/routing/url/UrlRouter.dart' as arch;
import 'package:flutterskel/userinformation/UserInformationRepository.dart' as arch;
import 'package:provider/provider.dart';

abstract class ProviderBuilder<
                  // Data
                  DataSourceClass,
                  // Routing
                  UrlRouterClass extends arch.UrlRouter,
                  PageRouterClass extends arch.PageRouter,
                  // Configuration
                  ConfigurationClass extends arch.Configuration,
                  // User Information
                  UserInformationRepositoryClass extends arch.UserInformationRepository,
                  // Theme Related: Repository -> Theme -> Fonts -> Dimensions/Colors
                  ThemeRepositoryClass extends arch.ThemeRepository,
                  // Strings
                  StringsClass extends arch.StringResources,
                  // Images & Icons
                  ImagesClass,
                  IconsClass> {

  // DataSource
  DataSourceClass _dataSource;
  DataSourceClass buildDataSource();
  DataSourceClass getDataSource() {
    if(_dataSource == null) {
      _dataSource = buildDataSource();
    }
    return _dataSource;
  }

  // Page Router
  PageRouterClass _pageRouter;
  PageRouterClass buildPageRouter();
  PageRouterClass getPageRouter() {
    if(_pageRouter == null) {
      _pageRouter = buildPageRouter();
    }
    return _pageRouter;
  }

  // Url Router
  UrlRouterClass _urlRouter;
  UrlRouterClass buildUrlRouter();
  UrlRouterClass getUrlRouter() {
    if(_urlRouter == null) {
      _urlRouter = buildUrlRouter();
    }
    return _urlRouter;
  }

  // Configuration
  ConfigurationClass _configuration;
  ConfigurationClass buildConfiguration();
  ConfigurationClass getConfiguration() {
    if(_configuration == null) {
      _configuration = buildConfiguration();
    }
    return _configuration;
  }

  // Images
  ImagesClass _images;
  ImagesClass buildImages();
  ImagesClass getImages() {
    if(_images == null) {
      _images = buildImages();
    }
    return _images;
  }

  // Strings
  StringsRepository _stringsRepository;
  StringsClass buildStringsResources();
  StringsRepository _getStringsRepository() {
    if(_stringsRepository == null) {
      _stringsRepository = StringsRepository<StringsClass>(
        strings: buildStringsResources()
      );
    }
    return _stringsRepository;
  }

  // User Information
  UserInformationRepositoryClass _userInformationRepository;
  UserInformationRepositoryClass buildUserInformationRepository();
  UserInformationRepositoryClass getUserInformationRepository() {
    if(_userInformationRepository == null) {
      _userInformationRepository = buildUserInformationRepository();
    }
    return _userInformationRepository;
  }

  // Icons
  IconsClass _icons;
  IconsClass buildIcons();
  IconsClass _getIcons() {
    if(_icons == null) {
      _icons = buildIcons();
    }
    return _icons;
  }

  // Theme
  ThemeRepositoryClass _themeRepository;
  ThemeRepositoryClass buildThemeRepository();
  ThemeRepositoryClass _getThemeRepository() {
    if(_themeRepository == null) {
      _themeRepository = buildThemeRepository();
    }
    return _themeRepository;
  }


  List<SingleChildCloneableWidget> get providers => [
    // DataSource
    Provider<DataSourceClass>.value(value: getDataSource()),
    Provider<UrlRouterClass>.value(value: getUrlRouter()),
    Provider<PageRouterClass>.value(value: getPageRouter()),
    Provider<ImagesClass>.value(value: getImages()),
    Provider<IconsClass>.value(value: _getIcons()),

    // Change Notifiers
    ChangeNotifierProvider<ConfigurationClass>.value(value: getConfiguration()),
    ChangeNotifierProvider<UserInformationRepositoryClass>.value(value: getUserInformationRepository()),
    ChangeNotifierProvider<StringsRepository>.value(value: _getStringsRepository()),
    ChangeNotifierProvider<ThemeRepositoryClass>.value(value: _getThemeRepository())
  ];

  MultiProvider build({@required Widget child}) {
    return MultiProvider(
      providers: providers,
      child: child,
    );
  }
}
