import 'package:flutter/material.dart';
import 'package:flutterskel/resources/colors/ColorResources.dart';
import 'package:flutterskel/resources/dimensions/DimensionResources.dart';

/// Base class for fonts
/// Provides basic Material Design textStyles
abstract class FontResources {
  // Colors
  ColorResources colors;

  // Dimensions
  DimensionResources dimensions;

  // Font Family
  String fontFamily;

  // Text Styles
  TextStyle styleHeadline1;
  TextStyle styleHeadline2;
  TextStyle styleHeadline3;
  TextStyle styleHeadline4;
  TextStyle styleHeadline5;
  TextStyle styleHeadline6;
  TextStyle styleSectionTitle;
  TextStyle styleSubtitle1;
  TextStyle styleSubtitle2;
  TextStyle styleBody1;
  TextStyle styleBody2;
  TextStyle styleButton;
  TextStyle styleCaption;
  TextStyle styleOverline;

  FontResources({
    // Colors
    @required this.colors,
    // Dimensions
    @required this.dimensions,

    // Font Family
    TextStyle baseTextStyle,
    this.fontFamily,
    TextTheme theme
  }) {
    TextStyle baseStyle = baseTextStyle != null ? baseTextStyle : (fontFamily != null ? TextStyle(fontFamily: fontFamily) : TextStyle());

    styleHeadline1 = baseStyle.copyWith(
        color: colors.textPrimary,
        fontWeight: FontWeight.w600,
        fontStyle: FontStyle.normal,
        letterSpacing: dimensions.fontLetterSpacingHeadline1,
        fontSize: dimensions.fontSizeHeadline1);
    styleHeadline2 = baseStyle.copyWith(
        color: colors.textPrimary,
        fontWeight: FontWeight.w400,
        fontStyle: FontStyle.normal,
        letterSpacing: dimensions.fontLetterSpacingHeadline2,
        fontSize: dimensions.fontSizeHeadline2);
    styleHeadline3 = baseStyle.copyWith(
        color: colors.textPrimary,
        fontWeight: FontWeight.w400,
        fontStyle: FontStyle.normal,
        letterSpacing: dimensions.fontLetterSpacingHeadline3,
        fontSize: dimensions.fontSizeHeadline3);
    styleHeadline4 = baseStyle.copyWith(
        color: colors.textPrimary,
        fontWeight: FontWeight.w400,
        fontStyle: FontStyle.normal,
        letterSpacing: dimensions.fontLetterSpacingHeadline4,
        fontSize: dimensions.fontSizeHeadline4);
    styleHeadline5 = baseStyle.copyWith(
        color: colors.textPrimary,
        fontWeight: FontWeight.w400,
        fontStyle: FontStyle.normal,
        letterSpacing: dimensions.fontLetterSpacingHeadline5,
        fontSize: dimensions.fontSizeHeadline5);
    styleHeadline6 = baseStyle.copyWith(
        color: colors.textPrimary,
        fontWeight: FontWeight.w400,
        fontStyle: FontStyle.normal,
        letterSpacing: dimensions.fontLetterSpacingHeadline6,
        fontSize: dimensions.fontSizeHeadline6);

    styleSectionTitle = baseStyle.copyWith(
        color: colors.textPrimary,
        fontWeight: FontWeight.w400,
        fontStyle: FontStyle.normal,
        letterSpacing: dimensions.fontLetterSpacingSectionTitle,
        fontSize: dimensions.fontSizeSectionTitle);

    styleSubtitle1 = baseStyle.copyWith(
        color: colors.textPrimary,
        fontWeight: FontWeight.w400,
        fontStyle: FontStyle.normal,
        letterSpacing: dimensions.fontLetterSpacingSubtitle1,
        fontSize: dimensions.fontSizeSubtitle1);
    styleSubtitle2 = baseStyle.copyWith(
        color: colors.textPrimary,
        fontWeight: FontWeight.w400,
        fontStyle: FontStyle.normal,
        letterSpacing: dimensions.fontLetterSpacingSubtitle2,
        fontSize: dimensions.fontSizeSubtitle2);

    styleBody1 = baseStyle.copyWith(
        color: colors.textPrimary,
        fontWeight: FontWeight.w400,
        fontStyle: FontStyle.normal,
        letterSpacing: dimensions.fontLetterSpacingBody1,
        fontSize: dimensions.fontSizeBody1);
    styleBody2 = baseStyle.copyWith(
        color: colors.textPrimary,
        fontWeight: FontWeight.w400,
        fontStyle: FontStyle.normal,
        letterSpacing: dimensions.fontLetterSpacingBody2,
        fontSize: dimensions.fontSizeBody2);

    styleButton = baseStyle.copyWith(
        color: colors.textPrimary,
        fontWeight: FontWeight.w400,
        fontStyle: FontStyle.normal,
        letterSpacing: dimensions.fontLetterSpacingButton,
        fontSize: dimensions.fontSizeButton);

    styleCaption = baseStyle.copyWith(
        color: colors.textPrimary,
        fontWeight: FontWeight.w400,
        fontStyle: FontStyle.normal,
        letterSpacing: dimensions.fontLetterSpacingCaption,
        fontSize: dimensions.fontSizeCaption);

    styleOverline = baseStyle.copyWith(
        color: colors.textPrimary,
        fontWeight: FontWeight.w500,
        fontStyle: FontStyle.normal,
        letterSpacing: dimensions.fontLetterSpacingOverline,
        fontSize: dimensions.fontSizeOverline);
  }
}
